
/******************************
 * login.css
 ******************************/

div.form {
    margin: auto;
    float: none;
    text-align: center;
}

div.field-wrapper {
    margin: 10px auto;
    width: 300px;
}

div.field-wrapper label {
    margin: 0px;
    text-align: right;
    width: 100px;
}

div.field-wrapper input {
    width: 150px;
}

div.buttons input {
    margin: auto 5px;
}
