
/******************************
 * Filters
 ******************************/

div.filters div.filter {
    margin-bottom: 10px;
}

div.filters div.filter label {
    margin-right: 8px;
}

div.filters div.filter select.filter-type {
    margin-right: 8px;
}

div.filters div.filter div.value {
    display: inline;
}

div.filters div.buttons * {
    margin-right: 8px;
}
