
/******************************
 * Permission Lists
 ******************************/

div.field-wrapper.permissions div.field div.group {
    margin-bottom: 10px;
}

div.field-wrapper.permissions div.field div.group p {
    font-weight: bold;
}

div.field-wrapper.permissions div.field label {
    float: none;
    font-weight: normal;
}

div.field-wrapper.permissions div.field label input {
    margin-left: 15px;
    margin-right: 10px;
}

div.field-wrapper.permissions div.field div.group p.perm {
    font-weight: normal;
    margin-left: 15px;
}

div.field-wrapper.permissions div.field div.group p.perm span {
    font-family: monospace;
    /* font-weight: bold; */
    margin-right: 10px;
}
