
/******************************
 * Main Layout
 ******************************/

html, body, #container {
    height: 100%;
}

body > #container {
    height: auto;
    min-height: 100%;
}

#container {
    margin: 0 auto;
    width: 1000px;
}

#header {
    border-bottom: 1px solid #000000;
    overflow: auto;
}

#body {
    padding-top: 15px;
    padding-bottom: 5em;
}

#footer {
    clear: both;
    margin-top: -4em;
    text-align: center;
}


/******************************
 * Header
 ******************************/

#header h1 {
    margin: 0px 5px 10px 5px;
}

#login {
    margin: 8px 20px auto auto;
}

#login a.username {
    font-weight: bold;
}

#user-menu {
    float: left;
}

#home-link {
    font-weight: bold;
}

#header-links {
    float: right;
    text-align: right;
}

#main-menu {
    border-top: 1px solid black;
    clear: both;
    font-weight: bold;
}

#main-menu li {
    display: inline;
    margin-right: 15px;
}
