
/******************************
 * Form Wrapper
 ******************************/

div.form-wrapper {
    overflow: auto;
}


/******************************
 * Context Menu
 ******************************/

div.form-wrapper ul.context-menu {
    float: right;
    list-style-type: none;
    margin: 0px;
    text-align: right;
}


/******************************
 * Forms
 ******************************/

div.form,
div.fieldset-form,
div.fieldset {
    float: left;
    margin-top: 10px;
}


/******************************
 * Fieldsets
 ******************************/

div.field-wrapper {
    clear: both;
    min-height: 30px;
    overflow: auto;
    padding: 5px;
}

div.field-wrapper.error {
    background-color: #ddcccc;
    border: 2px solid #dd6666;
}

div.field-wrapper label {
    color: #000000;
    display: block;
    float: left;
    width: 140px;
    font-weight: bold;
    margin-top: 2px;
    white-space: nowrap;
}

div.field-wrapper div.field-error {
    /* clear: both; */
    color: #dd6666;
    font-weight: bold;
}

div.field-wrapper div.field {
    display: block;
    float: left;
    margin-bottom: 5px;
    line-height: 25px;
}

div.field-wrapper div.field input[type=text],
div.field-wrapper div.field input[type=password],
div.field-wrapper div.field select,
div.field-wrapper div.field textarea {
    width: 320px;
}

div.buttons {
    clear: both;
    margin-top: 10px;
}

div.buttons * {
    margin-right: 8px;
}
