
/******************************
 * General
 ******************************/

* {
    margin: 0px;
}

html, body {
    font-family: sans-serif;
    font-size: 11pt;
}

a {
    text-decoration: none;
}

a:hover {
    text-decoration: underline;
}

h1 {
    margin-bottom: 15px;
}

h2 {
    font-size: 12pt;
    margin: 20px auto 10px auto;
}

li {
    line-height: 2em;
}

p {
    margin-bottom: 5px;
}

.left {
    float: left;
    text-align: left;
}

.right {
    float: right;
    text-align: right;
}

.wrapper {
    overflow: auto;
}

div.buttons {
    clear: both;
    margin-top: 10px;
}

/* div.controls { */
/*     font-weight: bold; */
/*     margin: 10px auto; */
/* } */

/* div.controls div { */
/*     margin: 5px; */
/* } */

/* div.controls label { */
/*     display: block; */
/*     float: left; */
/*     width: 120px; */
/* } */

div.dialog {
    display: none;
}

div.flash-message {
    background-color: #dddddd;
    margin-bottom: 8px;
    padding: 3px;
}

div.error {
    color: #dd6666;
    font-weight: bold;
    margin-bottom: 10px;
}

ul.error {
    color: #dd6666;
    font-weight: bold;
    padding: 0px;
}

ul.error li {
    list-style-type: none;
}
